import { MessagePublisherPort } from '@mobicoop/ddd-library';
import { AD_MESSAGE_PUBLISHER } from '@modules/ad/ad.di-tokens';
import { Inject, Injectable } from '@nestjs/common';
import { OnEvent } from '@nestjs/event-emitter';
import { AD_DELETED_ROUTING_KEY } from '@src/app.constants';
import { AdDeletedDomainEvent } from '../../domain/events/ad-delete.domain-event';

@Injectable()
export class PublishMessageWhenAdIsDeletedDomainEventHandler {
  constructor(
    @Inject(AD_MESSAGE_PUBLISHER)
    private readonly messagePublisher: MessagePublisherPort,
  ) {}

  @OnEvent(AdDeletedDomainEvent.name, { async: true, promisify: true })
  async handle(event: AdDeletedDomainEvent): Promise<void> {
    this.messagePublisher.publish(
      AD_DELETED_ROUTING_KEY,
      JSON.stringify(event),
    );
  }
}
