import { UserDeletedMessageHandler } from '@modules/ad/interface/message-handlers/user-deleted.message-handler';
import { CommandBus } from '@nestjs/cqrs';
import { Test, TestingModule } from '@nestjs/testing';

const mockCommandBus = {
  execute: jest.fn(),
};

describe('Matcher Ad Created Message Handler', () => {
  let userDeletedMessageHandler: UserDeletedMessageHandler;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: CommandBus,
          useValue: mockCommandBus,
        },
        UserDeletedMessageHandler,
      ],
    }).compile();

    userDeletedMessageHandler = module.get<UserDeletedMessageHandler>(
      UserDeletedMessageHandler,
    );
  });

  afterEach(async () => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(userDeletedMessageHandler).toBeDefined();
  });

  it('should call the command bus', async () => {
    const userId = '4eb6a6af-ecfd-41c3-9118-473a507014d4';
    const userDeletedMessage = `{"id":"${userId}"}`;
    await userDeletedMessageHandler.userDeleted(userDeletedMessage);
    expect(mockCommandBus.execute).toHaveBeenCalledTimes(1);
    expect(mockCommandBus.execute.mock.lastCall[0].id).toBe(userId);
  });
});
